@@ -1,13 +1,15 @@
 from __future__ import annotations
 
 from enum import IntEnum
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
-from docutils.statemachine import StringList
-
-from sphinx.application import Sphinx
 from sphinx.ext.autodoc import ClassDocumenter, bool_option
 
+if TYPE_CHECKING:
+    from docutils.statemachine import StringList
+
+    from sphinx.application import Sphinx
+
 
 class IntEnumDocumenter(ClassDocumenter):
     objtype = 'intenum'
