@@ -1,25 +1,33 @@
-# -*- coding: utf-8 -*-
-"""
-    sphinx.versioning
-    ~~~~~~~~~~~~~~~~~
+"""Implements the low-level algorithms Sphinx uses for versioning doctrees."""
+from __future__ import annotations
 
-    Implements the low-level algorithms Sphinx uses for the versioning of
-    doctrees.
-
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-from uuid import uuid4
+import pickle
+from itertools import product, zip_longest
 from operator import itemgetter
+from os import path
+from typing import TYPE_CHECKING, Any
+from uuid import uuid4
 
-from sphinx.util.pycompat import product, zip_longest, all
+from sphinx.transforms import SphinxTransform
 
+if TYPE_CHECKING:
+    from collections.abc import Iterator
+
+    from docutils.nodes import Node
+
+    from sphinx.application import Sphinx
+
+try:
+    import Levenshtein
+    IS_SPEEDUP = True
+except ImportError:
+    IS_SPEEDUP = False
 
 # anything below that ratio is considered equal/changed
 VERSIONING_RATIO = 65
 
 
-def add_uids(doctree, condition):
+def add_uids(doctree: Node, condition: Any) -> Iterator[Node]:
     """Add a unique id to every node in the `doctree` which matches the
     condition and yield the nodes.
 
@@ -29,12 +37,12 @@ def add_uids(doctree, condition):
     :param condition:
         A callable which returns either ``True`` or ``False`` for a given node.
     """
-    for node in doctree.traverse(condition):
+    for node in doctree.findall(condition):
         node.uid = uuid4().hex
         yield node
 
 
-def merge_doctrees(old, new, condition):
+def merge_doctrees(old: Node, new: Node, condition: Any) -> Iterator[Node]:
     """Merge the `old` doctree with the `new` one while looking at nodes
     matching the `condition`.
 
@@ -44,8 +52,8 @@ def merge_doctrees(old, new, condition):
     :param condition:
         A callable which returns either ``True`` or ``False`` for a given node.
     """
-    old_iter = old.traverse(condition)
-    new_iter = new.traverse(condition)
+    old_iter = old.findall(condition)
+    new_iter = new.findall(condition)
     old_nodes = []
     new_nodes = []
     ratios = {}
@@ -55,6 +63,9 @@ def merge_doctrees(old, new, condition):
         if old_node is None:
             new_nodes.append(new_node)
             continue
+        if not getattr(old_node, 'uid', None):
+            # maybe config.gettext_uuid has been changed.
+            old_node.uid = uuid4().hex
         if new_node is None:
             old_nodes.append(old_node)
             continue
@@ -80,7 +91,7 @@ def merge_doctrees(old, new, condition):
     # choose the old node with the best ratio for each new node and set the uid
     # as long as the ratio is under a certain value, in which case we consider
     # them not changed but different
-    ratios = sorted(ratios.iteritems(), key=itemgetter(1))
+    ratios = sorted(ratios.items(), key=itemgetter(1))  # type: ignore[assignment]
     for (old_node, new_node), ratio in ratios:
         if new_node in seen:
             continue
@@ -98,16 +109,20 @@ def merge_doctrees(old, new, condition):
         yield new_node
 
 
-def get_ratio(old, new):
-    """Return a "similiarity ratio" (in percent) representing the similarity
+def get_ratio(old: str, new: str) -> float:
+    """Return a "similarity ratio" (in percent) representing the similarity
     between the two strings where 0 is equal and anything above less than equal.
     """
     if not all([old, new]):
         return VERSIONING_RATIO
-    return levenshtein_distance(old, new) / (len(old) / 100.0)
+
+    if IS_SPEEDUP:
+        return Levenshtein.distance(old, new) / (len(old) / 100.0)
+    else:
+        return levenshtein_distance(old, new) / (len(old) / 100.0)
 
 
-def levenshtein_distance(a, b):
+def levenshtein_distance(a: str, b: str) -> int:
     """Return the Levenshtein edit distance between two strings *a* and *b*."""
     if a == b:
         return 0
@@ -115,7 +130,7 @@ def levenshtein_distance(a, b):
         a, b = b, a
     if not a:
         return len(b)
-    previous_row = xrange(len(b) + 1)
+    previous_row = list(range(len(b) + 1))
     for i, column1 in enumerate(a):
         current_row = [i + 1]
         for j, column2 in enumerate(b):
@@ -125,3 +140,39 @@ def levenshtein_distance(a, b):
             current_row.append(min(insertions, deletions, substitutions))
         previous_row = current_row
     return previous_row[-1]
+
+
+class UIDTransform(SphinxTransform):
+    """Add UIDs to doctree for versioning."""
+    default_priority = 880
+
+    def apply(self, **kwargs: Any) -> None:
+        env = self.env
+        old_doctree = None
+        if not env.versioning_condition:
+            return
+
+        if env.versioning_compare:
+            # get old doctree
+            try:
+                filename = path.join(env.doctreedir, env.docname + '.doctree')
+                with open(filename, 'rb') as f:
+                    old_doctree = pickle.load(f)
+            except OSError:
+                pass
+
+        # add uids for versioning
+        if not env.versioning_compare or old_doctree is None:
+            list(add_uids(self.document, env.versioning_condition))
+        else:
+            list(merge_doctrees(old_doctree, self.document, env.versioning_condition))
+
+
+def setup(app: Sphinx) -> dict[str, Any]:
+    app.add_transform(UIDTransform)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
