@@ -11,7 +11,6 @@ from docutils import nodes
 
 import sphinx.application
 from sphinx.errors import ExtensionError
-from sphinx.testing.path import path
 from sphinx.testing.util import SphinxTestApp, strip_escseq
 from sphinx.util import logging
 
@@ -24,13 +23,11 @@ def test_instantiation(tmp_path_factory, rootdir: str, monkeypatch):
     if rootdir and not src_dir.exists():
         shutil.copytree(Path(str(rootdir)) / 'test-root', src_dir)
 
-    monkeypatch.setattr('sphinx.application.abspath', lambda x: x)
-
     syspath = sys.path[:]
 
     # When
     app_ = SphinxTestApp(
-        srcdir=path(src_dir),
+        srcdir=src_dir,
         status=StringIO(),
         warning=StringIO(),
     )
