@@ -29,12 +29,12 @@ def test_catalog_info_for_sub_domain_file_and_path():
     assert cat.mo_path == os.path.join('path', 'sub/domain.mo')
 
 
-def test_catalog_outdated(tempdir):
-    (tempdir / 'test.po').write_text('#', encoding='utf8')
-    cat = i18n.CatalogInfo(tempdir, 'test', 'utf-8')
+def test_catalog_outdated(tmp_path):
+    (tmp_path / 'test.po').write_text('#', encoding='utf8')
+    cat = i18n.CatalogInfo(tmp_path, 'test', 'utf-8')
     assert cat.is_outdated()  # if mo is not exist
 
-    mo_file = (tempdir / 'test.mo')
+    mo_file = (tmp_path / 'test.mo')
     mo_file.write_text('#', encoding='utf8')
     assert not cat.is_outdated()  # if mo is exist and newer than po
 
@@ -42,9 +42,9 @@ def test_catalog_outdated(tempdir):
     assert cat.is_outdated()  # if mo is exist and older than po
 
 
-def test_catalog_write_mo(tempdir):
-    (tempdir / 'test.po').write_text('#', encoding='utf8')
-    cat = i18n.CatalogInfo(tempdir, 'test', 'utf-8')
+def test_catalog_write_mo(tmp_path):
+    (tmp_path / 'test.po').write_text('#', encoding='utf8')
+    cat = i18n.CatalogInfo(tmp_path, 'test', 'utf-8')
     cat.write_mo('en')
     assert os.path.exists(cat.mo_path)
     with open(cat.mo_path, 'rb') as f:
@@ -67,7 +67,7 @@ def test_format_date():
     assert i18n.format_date(format, date=date, language='en') == format
 
     format = '%B %d, %Y, %H:%M:%S %I %p'
-    datet = datetime.datetime(2016, 2, 7, 5, 11, 17, 0)
+    datet = datetime.datetime(2016, 2, 7, 5, 11, 17, 0)  # NoQA: DTZ001
     assert i18n.format_date(format, date=datet, language='en') == 'February 07, 2016, 05:11:17 05 AM'
 
     format = '%B %-d, %Y, %-H:%-M:%-S %-I %p'
@@ -94,7 +94,6 @@ def test_format_date():
     assert i18n.format_date(format, date=datet, language='en') == '+0000'
 
 
-@pytest.mark.xfail(os.name != 'posix', reason="Path separators don't match on windows")
 def test_get_filename_for_language(app):
     app.env.temp_data['docname'] = 'index'
 
@@ -147,45 +146,45 @@ def test_get_filename_for_language(app):
             '/subdir/en/foo.png')
 
 
-def test_CatalogRepository(tempdir):
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES').makedirs()
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / 'test1.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / 'test2.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / 'sub').makedirs()
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / 'sub' / 'test3.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / 'sub' / 'test4.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / '.dotdir').makedirs()
-    (tempdir / 'loc1' / 'xx' / 'LC_MESSAGES' / '.dotdir' / 'test5.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc1' / 'yy' / 'LC_MESSAGES').makedirs()
-    (tempdir / 'loc1' / 'yy' / 'LC_MESSAGES' / 'test6.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc2' / 'xx' / 'LC_MESSAGES').makedirs()
-    (tempdir / 'loc2' / 'xx' / 'LC_MESSAGES' / 'test1.po').write_text('#', encoding='utf8')
-    (tempdir / 'loc2' / 'xx' / 'LC_MESSAGES' / 'test7.po').write_text('#', encoding='utf8')
+def test_CatalogRepository(tmp_path):
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES').mkdir(parents=True, exist_ok=True)
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / 'test1.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / 'test2.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / 'sub').mkdir(parents=True, exist_ok=True)
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / 'sub' / 'test3.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / 'sub' / 'test4.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / '.dotdir').mkdir(parents=True, exist_ok=True)
+    (tmp_path / 'loc1' / 'xx' / 'LC_MESSAGES' / '.dotdir' / 'test5.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc1' / 'yy' / 'LC_MESSAGES').mkdir(parents=True, exist_ok=True)
+    (tmp_path / 'loc1' / 'yy' / 'LC_MESSAGES' / 'test6.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc2' / 'xx' / 'LC_MESSAGES').mkdir(parents=True, exist_ok=True)
+    (tmp_path / 'loc2' / 'xx' / 'LC_MESSAGES' / 'test1.po').write_text('#', encoding='utf8')
+    (tmp_path / 'loc2' / 'xx' / 'LC_MESSAGES' / 'test7.po').write_text('#', encoding='utf8')
 
     # for language xx
-    repo = i18n.CatalogRepository(tempdir, ['loc1', 'loc2'], 'xx', 'utf-8')
-    assert list(repo.locale_dirs) == [str(tempdir / 'loc1'),
-                                      str(tempdir / 'loc2')]
+    repo = i18n.CatalogRepository(tmp_path, ['loc1', 'loc2'], 'xx', 'utf-8')
+    assert list(repo.locale_dirs) == [str(tmp_path / 'loc1'),
+                                      str(tmp_path / 'loc2')]
     assert all(isinstance(c, i18n.CatalogInfo) for c in repo.catalogs)
     assert sorted(c.domain for c in repo.catalogs) == ['sub/test3', 'sub/test4',
                                                        'test1', 'test1', 'test2', 'test7']
 
     # for language yy
-    repo = i18n.CatalogRepository(tempdir, ['loc1', 'loc2'], 'yy', 'utf-8')
+    repo = i18n.CatalogRepository(tmp_path, ['loc1', 'loc2'], 'yy', 'utf-8')
     assert sorted(c.domain for c in repo.catalogs) == ['test6']
 
     # unknown languages
-    repo = i18n.CatalogRepository(tempdir, ['loc1', 'loc2'], 'zz', 'utf-8')
+    repo = i18n.CatalogRepository(tmp_path, ['loc1', 'loc2'], 'zz', 'utf-8')
     assert sorted(c.domain for c in repo.catalogs) == []
 
     # no languages
-    repo = i18n.CatalogRepository(tempdir, ['loc1', 'loc2'], None, 'utf-8')
+    repo = i18n.CatalogRepository(tmp_path, ['loc1', 'loc2'], None, 'utf-8')
     assert sorted(c.domain for c in repo.catalogs) == []
 
     # unknown locale_dirs
-    repo = i18n.CatalogRepository(tempdir, ['loc3'], None, 'utf-8')
+    repo = i18n.CatalogRepository(tmp_path, ['loc3'], None, 'utf-8')
     assert sorted(c.domain for c in repo.catalogs) == []
 
     # no locale_dirs
-    repo = i18n.CatalogRepository(tempdir, [], None, 'utf-8')
+    repo = i18n.CatalogRepository(tmp_path, [], None, 'utf-8')
     assert sorted(c.domain for c in repo.catalogs) == []
