@@ -11,8 +11,14 @@ from docutils.parsers import rst
 from docutils.utils import new_document
 
 from sphinx.transforms import ApplySourceWorkaround
-from sphinx.util.nodes import (NodeMatcher, apply_source_workaround, clean_astext,
-                               extract_messages, make_id, split_explicit_title)
+from sphinx.util.nodes import (
+    NodeMatcher,
+    apply_source_workaround,
+    clean_astext,
+    extract_messages,
+    make_id,
+    split_explicit_title,
+)
 
 
 def _transform(doctree):
@@ -87,7 +93,7 @@ def test_NodeMatcher():
 
 
 @pytest.mark.parametrize(
-    'rst,node_cls,count',
+    ('rst', 'node_cls', 'count'),
     [
         (
             """
@@ -95,7 +101,7 @@ def test_NodeMatcher():
 
               admonition body
            """,
-            nodes.title, 1
+            nodes.title, 1,
         ),
         (
             """
@@ -138,7 +144,7 @@ def test_NodeMatcher():
             nodes.line, 2,
 
         ),
-    ]
+    ],
 )
 def test_extract_messages(rst, node_cls, count):
     msg = extract_messages(_get_doctree(dedent(rst)))
@@ -181,7 +187,7 @@ def test_clean_astext():
 
 
 @pytest.mark.parametrize(
-    'prefix, term, expected',
+    ('prefix', 'term', 'expected'),
     [
         ('', '', 'id0'),
         ('term', '', 'term-0'),
@@ -214,7 +220,7 @@ def test_make_id_sequential(app):
 
 
 @pytest.mark.parametrize(
-    'title, expected',
+    ('title', 'expected'),
     [
         # implicit
         ('hello', (False, 'hello', 'hello')),
@@ -222,7 +228,7 @@ def test_make_id_sequential(app):
         ('hello <world>', (True, 'hello', 'world')),
         # explicit (title having angle brackets)
         ('hello <world> <sphinx>', (True, 'hello <world>', 'sphinx')),
-    ]
+    ],
 )
 def test_split_explicit_target(title, expected):
     assert expected == split_explicit_title(title)
