@@ -1,6 +1,7 @@
 """Tests util.typing functions."""
 
 import sys
+from enum import Enum
 from numbers import Integral
 from struct import Struct
 from types import TracebackType
@@ -20,7 +21,7 @@ from typing import (
 import pytest
 
 from sphinx.ext.autodoc import mock
-from sphinx.util.typing import restify, stringify_annotation
+from sphinx.util.typing import INVALID_BUILTIN_CLASSES, restify, stringify_annotation
 
 
 class MyClass1:
@@ -31,6 +32,10 @@ class MyClass2(MyClass1):
     __qualname__ = '<MyClass2>'
 
 
+class MyEnum(Enum):
+    a = 1
+
+
 T = TypeVar('T')
 MyInt = NewType('MyInt', int)
 
@@ -69,6 +74,15 @@ def test_restify():
     assert restify('str', "smart") == "str"
 
 
+def test_is_invalid_builtin_class():
+    # if these tests start failing, it means that the __module__
+    # of one of these classes has changed, and INVALID_BUILTIN_CLASSES
+    # in sphinx.util.typing needs to be updated.
+    assert INVALID_BUILTIN_CLASSES.keys() == {Struct, TracebackType}
+    assert Struct.__module__ == '_struct'
+    assert TracebackType.__module__ == 'builtins'
+
+
 def test_restify_type_hints_containers():
     assert restify(List) == ":py:class:`~typing.List`"
     assert restify(Dict) == ":py:class:`~typing.Dict`"
@@ -144,6 +158,9 @@ def test_restify_type_hints_typevars():
     assert restify(List[T]) == ":py:class:`~typing.List`\\ [:py:obj:`tests.test_util_typing.T`]"
     assert restify(List[T], "smart") == ":py:class:`~typing.List`\\ [:py:obj:`~tests.test_util_typing.T`]"
 
+    assert restify(list[T]) == ":py:class:`list`\\ [:py:obj:`tests.test_util_typing.T`]"
+    assert restify(list[T], "smart") == ":py:class:`list`\\ [:py:obj:`~tests.test_util_typing.T`]"
+
     if sys.version_info[:2] >= (3, 10):
         assert restify(MyInt) == ":py:class:`tests.test_util_typing.MyInt`"
         assert restify(MyInt, "smart") == ":py:class:`~tests.test_util_typing.MyInt`"
@@ -162,36 +179,60 @@ def test_restify_type_hints_custom_class():
 
 def test_restify_type_hints_alias():
     MyStr = str
-    MyTuple = Tuple[str, str]
+    MyTypingTuple = Tuple[str, str]
+    MyTuple = tuple[str, str]
     assert restify(MyStr) == ":py:class:`str`"
-    assert restify(MyTuple) == ":py:class:`~typing.Tuple`\\ [:py:class:`str`, :py:class:`str`]"
+    assert restify(MyTypingTuple) == ":py:class:`~typing.Tuple`\\ [:py:class:`str`, :py:class:`str`]"
+    assert restify(MyTuple) == ":py:class:`tuple`\\ [:py:class:`str`, :py:class:`str`]"
 
 
 def test_restify_type_ForwardRef():
-    from typing import ForwardRef  # type: ignore
-    assert restify(ForwardRef("myint")) == ":py:class:`myint`"
+    from typing import ForwardRef  # type: ignore[attr-defined]
+    assert restify(ForwardRef("MyInt")) == ":py:class:`MyInt`"
+
+    assert restify(list[ForwardRef("MyInt")]) == ":py:class:`list`\\ [:py:class:`MyInt`]"
+
+    assert restify(Tuple[dict[ForwardRef("MyInt"), str], list[List[int]]]) == ":py:class:`~typing.Tuple`\\ [:py:class:`dict`\\ [:py:class:`MyInt`, :py:class:`str`], :py:class:`list`\\ [:py:class:`~typing.List`\\ [:py:class:`int`]]]"  # type: ignore[attr-defined]
 
 
 def test_restify_type_Literal():
-    from typing import Literal  # type: ignore
+    from typing import Literal  # type: ignore[attr-defined]
     assert restify(Literal[1, "2", "\r"]) == ":py:obj:`~typing.Literal`\\ [1, '2', '\\r']"
 
+    assert restify(Literal[MyEnum.a], 'fully-qualified-except-typing') == ':py:obj:`~typing.Literal`\\ [:py:attr:`tests.test_util_typing.MyEnum.a`]'
+    assert restify(Literal[MyEnum.a], 'smart') == ':py:obj:`~typing.Literal`\\ [:py:attr:`~tests.test_util_typing.MyEnum.a`]'
+
 
-@pytest.mark.skipif(sys.version_info[:2] <= (3, 8), reason='python 3.9+ is required.')
 def test_restify_pep_585():
-    assert restify(list[str]) == ":py:class:`list`\\ [:py:class:`str`]"  # type: ignore
-    assert restify(dict[str, str]) == (":py:class:`dict`\\ "  # type: ignore
+    assert restify(list[str]) == ":py:class:`list`\\ [:py:class:`str`]"  # type: ignore[attr-defined]
+    assert restify(dict[str, str]) == (":py:class:`dict`\\ "  # type: ignore[attr-defined]
                                        "[:py:class:`str`, :py:class:`str`]")
-    assert restify(dict[str, tuple[int, ...]]) == (":py:class:`dict`\\ "  # type: ignore
+    assert restify(tuple[str, ...]) == ":py:class:`tuple`\\ [:py:class:`str`, ...]"
+    assert restify(tuple[str, str, str]) == (":py:class:`tuple`\\ "
+                                             "[:py:class:`str`, :py:class:`str`, "
+                                             ":py:class:`str`]")
+    assert restify(dict[str, tuple[int, ...]]) == (":py:class:`dict`\\ "  # type: ignore[attr-defined]
                                                    "[:py:class:`str`, :py:class:`tuple`\\ "
                                                    "[:py:class:`int`, ...]]")
 
+    assert restify(tuple[()]) == ":py:class:`tuple`\\ [()]"
+
+    # Mix old typing with PEP 585
+    assert restify(List[dict[str, Tuple[str, ...]]]) == (":py:class:`~typing.List`\\ "
+                                                         "[:py:class:`dict`\\ "
+                                                         "[:py:class:`str`, :py:class:`~typing.Tuple`\\ "
+                                                         "[:py:class:`str`, ...]]]")
+    assert restify(tuple[MyList[list[int]], int]) == (":py:class:`tuple`\\ ["
+                                                      ":py:class:`tests.test_util_typing.MyList`\\ "
+                                                      "[:py:class:`list`\\ [:py:class:`int`]], "
+                                                      ":py:class:`int`]")
+
 
 @pytest.mark.skipif(sys.version_info[:2] <= (3, 9), reason='python 3.10+ is required.')
 def test_restify_type_union_operator():
-    assert restify(int | None) == ":py:class:`int` | :py:obj:`None`"  # type: ignore
-    assert restify(int | str) == ":py:class:`int` | :py:class:`str`"  # type: ignore
-    assert restify(int | str | None) == (":py:class:`int` | :py:class:`str` | "  # type: ignore
+    assert restify(int | None) == ":py:class:`int` | :py:obj:`None`"  # type: ignore[attr-defined]
+    assert restify(int | str) == ":py:class:`int` | :py:class:`str`"  # type: ignore[attr-defined]
+    assert restify(int | str | None) == (":py:class:`int` | :py:class:`str` | "  # type: ignore[attr-defined]
                                          ":py:obj:`None`")
 
 
@@ -283,7 +324,6 @@ def test_stringify_type_hints_containers():
     assert stringify_annotation(Generator[None, None, None], "smart") == "~typing.Generator[None, None, None]"
 
 
-@pytest.mark.skipif(sys.version_info[:2] <= (3, 8), reason='python 3.9+ is required.')
 def test_stringify_type_hints_pep_585():
     assert stringify_annotation(list[int], 'fully-qualified-except-typing') == "list[int]"
     assert stringify_annotation(list[int], "smart") == "list[int]"
@@ -306,13 +346,20 @@ def test_stringify_type_hints_pep_585():
     assert stringify_annotation(list[dict[str, tuple]], 'fully-qualified-except-typing') == "list[dict[str, tuple]]"
     assert stringify_annotation(list[dict[str, tuple]], "smart") == "list[dict[str, tuple]]"
 
+    assert stringify_annotation(MyList[tuple[int, int]], 'fully-qualified-except-typing') == "tests.test_util_typing.MyList[tuple[int, int]]"
+    assert stringify_annotation(MyList[tuple[int, int]], "fully-qualified") == "tests.test_util_typing.MyList[tuple[int, int]]"
+    assert stringify_annotation(MyList[tuple[int, int]], "smart") == "~tests.test_util_typing.MyList[tuple[int, int]]"
+
     assert stringify_annotation(type[int], 'fully-qualified-except-typing') == "type[int]"
     assert stringify_annotation(type[int], "smart") == "type[int]"
 
+    # Mix typing and pep 585
+    assert stringify_annotation(tuple[List[dict[int, str]], str, ...], 'fully-qualified-except-typing') == "tuple[List[dict[int, str]], str, ...]"
+    assert stringify_annotation(tuple[List[dict[int, str]], str, ...], "smart") == "tuple[~typing.List[dict[int, str]], str, ...]"
+
 
-@pytest.mark.skipif(sys.version_info[:2] <= (3, 8), reason='python 3.9+ is required.')
 def test_stringify_Annotated():
-    from typing import Annotated  # type: ignore
+    from typing import Annotated  # type: ignore[attr-defined]
     assert stringify_annotation(Annotated[str, "foo", "bar"], 'fully-qualified-except-typing') == "str"
     assert stringify_annotation(Annotated[str, "foo", "bar"], "smart") == "str"
 
@@ -330,10 +377,18 @@ def test_stringify_type_hints_string():
     assert stringify_annotation(List["int"], 'fully-qualified') == "typing.List[int]"
     assert stringify_annotation(List["int"], "smart") == "~typing.List[int]"
 
+    assert stringify_annotation(list["int"], 'fully-qualified-except-typing') == "list[int]"
+    assert stringify_annotation(list["int"], 'fully-qualified') == "list[int]"
+    assert stringify_annotation(list["int"], "smart") == "list[int]"
+
     assert stringify_annotation("Tuple[str]", 'fully-qualified-except-typing') == "Tuple[str]"
     assert stringify_annotation("Tuple[str]", 'fully-qualified') == "Tuple[str]"
     assert stringify_annotation("Tuple[str]", "smart") == "Tuple[str]"
 
+    assert stringify_annotation("tuple[str]", 'fully-qualified-except-typing') == "tuple[str]"
+    assert stringify_annotation("tuple[str]", 'fully-qualified') == "tuple[str]"
+    assert stringify_annotation("tuple[str]", "smart") == "tuple[str]"
+
     assert stringify_annotation("unknown", 'fully-qualified-except-typing') == "unknown"
     assert stringify_annotation("unknown", 'fully-qualified') == "unknown"
     assert stringify_annotation("unknown", "smart") == "unknown"
@@ -395,6 +450,9 @@ def test_stringify_type_hints_typevars():
     assert stringify_annotation(List[T], 'fully-qualified-except-typing') == "List[tests.test_util_typing.T]"
     assert stringify_annotation(List[T], "smart") == "~typing.List[~tests.test_util_typing.T]"
 
+    assert stringify_annotation(list[T], 'fully-qualified-except-typing') == "list[tests.test_util_typing.T]"
+    assert stringify_annotation(list[T], "smart") == "list[~tests.test_util_typing.T]"
+
     if sys.version_info[:2] >= (3, 10):
         assert stringify_annotation(MyInt, 'fully-qualified-except-typing') == "tests.test_util_typing.MyInt"
         assert stringify_annotation(MyInt, "smart") == "~tests.test_util_typing.MyInt"
@@ -418,27 +476,37 @@ def test_stringify_type_hints_alias():
     assert stringify_annotation(MyStr, 'fully-qualified-except-typing') == "str"
     assert stringify_annotation(MyStr, "smart") == "str"
 
-    assert stringify_annotation(MyTuple) == "Tuple[str, str]"  # type: ignore
-    assert stringify_annotation(MyTuple, "smart") == "~typing.Tuple[str, str]"  # type: ignore
+    assert stringify_annotation(MyTuple) == "Tuple[str, str]"  # type: ignore[attr-defined]
+    assert stringify_annotation(MyTuple, "smart") == "~typing.Tuple[str, str]"  # type: ignore[attr-defined]
 
 
 def test_stringify_type_Literal():
-    from typing import Literal  # type: ignore
+    from typing import Literal  # type: ignore[attr-defined]
     assert stringify_annotation(Literal[1, "2", "\r"], 'fully-qualified-except-typing') == "Literal[1, '2', '\\r']"
     assert stringify_annotation(Literal[1, "2", "\r"], "fully-qualified") == "typing.Literal[1, '2', '\\r']"
     assert stringify_annotation(Literal[1, "2", "\r"], "smart") == "~typing.Literal[1, '2', '\\r']"
 
+    assert stringify_annotation(Literal[MyEnum.a], 'fully-qualified-except-typing') == 'Literal[tests.test_util_typing.MyEnum.a]'
+    assert stringify_annotation(Literal[MyEnum.a], 'fully-qualified') == 'typing.Literal[tests.test_util_typing.MyEnum.a]'
+    assert stringify_annotation(Literal[MyEnum.a], 'smart') == '~typing.Literal[MyEnum.a]'
+
 
 @pytest.mark.skipif(sys.version_info[:2] <= (3, 9), reason='python 3.10+ is required.')
 def test_stringify_type_union_operator():
-    assert stringify_annotation(int | None) == "int | None"  # type: ignore
-    assert stringify_annotation(int | None, "smart") == "int | None"  # type: ignore
+    assert stringify_annotation(int | None) == "int | None"  # type: ignore[attr-defined]
+    assert stringify_annotation(int | None, "smart") == "int | None"  # type: ignore[attr-defined]
 
-    assert stringify_annotation(int | str) == "int | str"  # type: ignore
-    assert stringify_annotation(int | str, "smart") == "int | str"  # type: ignore
+    assert stringify_annotation(int | str) == "int | str"  # type: ignore[attr-defined]
+    assert stringify_annotation(int | str, "smart") == "int | str"  # type: ignore[attr-defined]
 
-    assert stringify_annotation(int | str | None) == "int | str | None"  # type: ignore
-    assert stringify_annotation(int | str | None, "smart") == "int | str | None"  # type: ignore
+    assert stringify_annotation(int | str | None) == "int | str | None"  # type: ignore[attr-defined]
+    assert stringify_annotation(int | str | None, "smart") == "int | str | None"  # type: ignore[attr-defined]
+
+    assert stringify_annotation(int | tuple[dict[str, int | None], list[int | str]] | None) == "int | tuple[dict[str, int | None], list[int | str]] | None"  # type: ignore[attr-defined]
+    assert stringify_annotation(int | tuple[dict[str, int | None], list[int | str]] | None, "smart") == "int | tuple[dict[str, int | None], list[int | str]] | None"  # type: ignore[attr-defined]
+
+    assert stringify_annotation(int | Struct) == "int | struct.Struct"  # type: ignore[attr-defined]
+    assert stringify_annotation(int | Struct, "smart") == "int | ~struct.Struct"  # type: ignore[attr-defined]
 
 
 def test_stringify_broken_type_hints():
@@ -452,3 +520,17 @@ def test_stringify_mock():
         assert stringify_annotation(unknown, 'fully-qualified-except-typing') == 'unknown'
         assert stringify_annotation(unknown.secret.Class, 'fully-qualified-except-typing') == 'unknown.secret.Class'
         assert stringify_annotation(unknown.secret.Class, "smart") == 'unknown.secret.Class'
+
+
+def test_stringify_type_ForwardRef():
+    from typing import ForwardRef  # type: ignore[attr-defined]
+
+    assert stringify_annotation(ForwardRef("MyInt")) == "MyInt"
+    assert stringify_annotation(ForwardRef("MyInt"), 'smart') == "MyInt"
+
+    assert stringify_annotation(list[ForwardRef("MyInt")]) == "list[MyInt]"
+    assert stringify_annotation(list[ForwardRef("MyInt")], 'smart') == "list[MyInt]"
+
+    assert stringify_annotation(Tuple[dict[ForwardRef("MyInt"), str], list[List[int]]]) == "Tuple[dict[MyInt, str], list[List[int]]]"  # type: ignore[attr-defined]
+    assert stringify_annotation(Tuple[dict[ForwardRef("MyInt"), str], list[List[int]]], 'fully-qualified-except-typing') == "Tuple[dict[MyInt, str], list[List[int]]]"  # type: ignore[attr-defined]
+    assert stringify_annotation(Tuple[dict[ForwardRef("MyInt"), str], list[List[int]]], 'smart') == "~typing.Tuple[dict[MyInt, str], list[~typing.List[int]]]"  # type: ignore[attr-defined]
